@@ -4,13 +4,7 @@
 #
 ################################################################################
 #
-#  $Revision: 28 $
-#  $Author: mhx $
-#  $Date: 2010/02/20 17:21:14 +0100 $
-#
-################################################################################
-#
-#  Version 3.x, Copyright (C) 2004-2009, Marcus Holland-Moritz.
+#  Version 3.x, Copyright (C) 2004-2013, Marcus Holland-Moritz.
 #  Version 2.x, Copyright (C) 2001, Paul Marquess.
 #  Version 1.x, Copyright (C) 1999, Kenneth Albanowski.
 #
@@ -34,7 +28,7 @@ sub all_files_in_dir
   my @files = grep { !-d && !/^\./ } readdir DIR;  # no dirs or hidden files
   closedir DIR;
 
-  return map { cat_file($dir, $_) } @files;
+  return map { cat_file($dir, $_) } sort @files;
 }
 
 sub parse_todo
@@ -85,7 +79,7 @@ sub parse_partspec
   while (<F>) {
     /[ \t]+$/ and warn "$file:$.: warning: trailing whitespace\n";
     if ($section eq 'implementation') {
-      m!//! && !m!(?:=~|s/).*//! && !m!(?:ht|f)tp://!
+      m!//! && !m!(?:=~|s/).*//! && !m!(?:ht|f)tp(?:s)://!
           and warn "$file:$.: warning: potential C++ comment\n";
     }
     /^##/ and next;
@@ -310,21 +304,46 @@ sub parse_embed
       }
       else {
         my @e = split /\s*\|\s*/, $line;
-        if( @e >= 3 ) {
-          my($flags, $ret, $name, @args) = @e;
+        if ( @e >= 3 ) {
+          my ($flags, $ret, $name, @args) = @e;
+          next unless $flags =~ /A/; # Skip non-public entries
+
+          # Skip entries marked as deprecated or unstable,
+          next if $flags =~ /[DM]/;
           if ($name =~ /^[^\W\d]\w*$/) {
             for (@args) {
               $_ = [trim_arg($_)];
             }
             ($ret) = trim_arg($ret);
-            push @func, {
-              name  => $name,
-              flags => { map { $_, 1 } $flags =~ /./g },
-              ret   => $ret,
-              args  => \@args,
-              cond  => ppcond(\@pps),
+            my $cond = $pps[-1];
+            if ($cond and $cond->{cur} and
+                $cond->{cur} =~ /\bdefined\(USE_CPERL\)/ and
+                $^V !~ /c$/) {
+              print "skip cperl-specific $name $flags\n" unless $SILENT;
+            }
+            elsif ($cond and @{$cond->{pre}} and
+                   !defined $cond->{cur} and
+                   @{$cond->{pre}}[-1] =~ /\bdefined\(USE_CPERL\)/ and
+                   $^V =~ /c$/) {
+              print "skip perl5-specific $name $flags\n" unless $SILENT;
+            }
+            else {
+              push @func, {
+                name  => $name,
+                flags => { map { $_, 1 } $flags =~ /./g },
+                ret   => $ret,
+                args  => \@args,
+                cond  => ppcond(\@pps),
+              }
             };
           }
+          # like
+          #    PL_parser->linestr
+          # which documents a struct entry rather than a function
+          elsif ($name =~ /^PL_parser->(\w*)$/) {
+            ;
+          }
+          # but warn about non names
           else {
             warn "mysterious name [$name] in $file, line $.\n";
           }
@@ -380,7 +399,7 @@ sub parse_version
   if ($ver =~ /^(\d+)\.(\d+)\.(\d+)$/) {
     return ($1, $2, $3);
   }
-  elsif ($ver !~ /^\d+\.[\d_]+$/) {
+  elsif ($ver !~ /^\d+\.\d{3}(?:_\d{2})?$/) {
     die "cannot parse version '$ver'\n";
   }
 
